@@ -23,6 +23,10 @@ _LOGGER = logging.getLogger(__name__)
 from aiokevoplus.const import (
     CLIENT_ID,
     CLIENT_SECRET,
+    COMMAND_STATUS_CANCELLED,
+    COMMAND_STATUS_COMPLETE,
+    COMMAND_STATUS_DELIVERED,
+    COMMAND_STATUS_PROCESSING,
     LOCK_STATE_JAM,
     LOCK_STATE_LOCK,
     LOCK_STATE_LOCK_JAM,
@@ -157,6 +161,21 @@ class KevoApi:
         """Generate a client nonce."""
         return base64.b64encode(secrets.token_bytes(64)).decode()
 
+    async def __get_headers(self):
+        """Retrieve the headers needed to make api calls."""
+        cnonce = self.__get_client_nonce()
+        snonce = await self.__get_server_nonce()
+
+        headers = {
+            "X-unikey-cnonce": cnonce,
+            "X-unikey-context": "Web",
+            "X-unikey-nonce": snonce,
+            "Authorization": "Bearer " + self._access_token,
+            "Accept": "application/json",
+        }
+
+        return headers
+
     async def async_refresh_token(self):
         """Refresh the access token."""
         client = httpx.AsyncClient()
@@ -179,47 +198,73 @@ class KevoApi:
     async def _api_post(self, url, body):
         """POST to the API."""
         client = httpx.AsyncClient()
-        cnonce = self.__get_client_nonce()
-        snonce = await self.__get_server_nonce()
 
         # Reauth if needed
         if self._expires_at < time.time() + 100:
             await self.async_refresh_token()
 
-        headers = {
-            "X-unikey-cnonce": cnonce,
-            "X-unikey-context": "Web",
-            "X-unikey-nonce": snonce,
-            "Authorization": "Bearer " + self._access_token,
-            "Accept": "application/json",
-        }
+        headers = await self.__get_headers()
 
         res = await client.post(
             UNIKEY_API_URL_BASE + url,
             headers=headers,
             json=body,
         )
-        res.raise_for_status()
+        try:
+            res.raise_for_status()
+        except httpx.HTTPStatusError as ex:
+            if ex.response.status_code == 403:
+                await self.async_refresh_token()
+                headers = await self.__get_headers()
+                res = await client.post(
+                    UNIKEY_API_URL_BASE + url,
+                    headers=headers,
+                    json=body,
+                )
+                try:
+                    res.raise_for_status()
+                except httpx.HTTPStatusError as retryex:
+                    if retryex.response.status_code == 403:
+                        raise KevoAuthError()
+                    else:
+                        raise
+            else:
+                raise
         return res.json()
 
     async def get_locks(self):
         """Retrieve the list of available locks."""
         client = httpx.AsyncClient()
-        cnonce = self.__get_client_nonce()
-        snonce = await self.__get_server_nonce()
+        headers = await self.__get_headers()
+
+        # Reauth if needed
+        if self._expires_at < time.time() + 100:
+            await self.async_refresh_token()
 
-        headers = {
-            "X-unikey-cnonce": cnonce,
-            "X-unikey-context": "Web",
-            "X-unikey-nonce": snonce,
-            "Authorization": "Bearer " + self._access_token,
-            "Accept": "application/json",
-        }
         res = await client.get(
             UNIKEY_API_URL_BASE + "/api/v2/users/" + self._user_id + "/locks",
             headers=headers,
         )
-        res.raise_for_status()
+        try:
+            res.raise_for_status()
+        except httpx.HTTPStatusError as ex:
+            if ex.response.status_code == 403:
+                await self.async_refresh_token()
+                headers = await self.__get_headers()
+                res = await client.get(
+                    UNIKEY_API_URL_BASE + "/api/v2/users/" + self._user_id + "/locks",
+                    headers=headers,
+                )
+                try:
+                    res.raise_for_status()
+                except httpx.HTTPStatusError as retryex:
+                    if retryex.response.status_code == 403:
+                        raise KevoAuthError()
+                    else:
+                        raise
+                raise KevoAuthError()
+            else:
+                raise
         json_response = res.json()
         lock_response = json_response["locks"]
         self._devices = []
@@ -350,6 +395,10 @@ class KevoApi:
                 if lock is not None:
                     lock.battery_level = message_body["batteryLevel"]
                     boltState = message_body["boltState"]
+                    command = message_body["command"]
+                    command_status = None
+                    if command is not None:
+                        command_status = command["status"]
                     if boltState == LOCK_STATE_LOCK:
                         lock.is_locked = True
                         lock.is_jammed = False
@@ -369,6 +418,23 @@ class KevoApi:
                         lock.is_jammed = None
                         lock.is_locked = None
 
+                    if command_status is not None:
+                        if command_status in (
+                            COMMAND_STATUS_COMPLETE,
+                            COMMAND_STATUS_CANCELLED,
+                        ):
+                            lock.is_locking = False
+                            lock.is_unlocking = False
+                        elif command_status in (
+                            COMMAND_STATUS_PROCESSING,
+                            COMMAND_STATUS_DELIVERED,
+                        ):
+                            if command["type"] == LOCK_STATE_LOCK:
+                                lock.is_locking = True
+                                lock.is_unlocking = False
+                            else:
+                                lock.is_locking = False
+                                lock.is_unlocking = True
                     for callback in self._callbacks:
                         try:
                             callback(lock)
@@ -423,11 +489,13 @@ class KevoLock:
         self._name = name
         self._firmware = firmware
         self._battery_level = battery_level
-        if state in ("Locked", "LockedJam"):
+        self._is_locking = False
+        self._is_unlocking = False
+        if state in ("Locked", "LockedBoltJam"):
             self._is_locked = True
         else:
             self._is_locked = False
-        if state in ("UnlockedJam", "LockedJam"):
+        if state in ("BoltJam", "UnlockedBoltJam", "LockedBoltJam"):
             self._is_jammed = True
         else:
             self._is_jammed = False
@@ -483,6 +551,26 @@ class KevoLock:
         """Update the jammed state."""
         self._is_jammed = value
 
+    @property
+    def is_locking(self):
+        """Retrieve the locking state."""
+        return self._is_locking
+
+    @is_locking.setter
+    def is_locking(self, value):
+        """Update the locking state."""
+        self._is_locking = value
+
+    @property
+    def is_unlocking(self):
+        """Retrieve the unlocking state."""
+        return self._is_unlocking
+
+    @is_unlocking.setter
+    def is_unlocking(self, value):
+        """Update the unlocking state."""
+        self._is_unlocking = value
+
     @property
     def brand(self):
         """Retrieve the lock brand."""
