@@ -11,3 +11,8 @@ LOCK_STATE_UNLOCK = 2
 LOCK_STATE_JAM = 8
 LOCK_STATE_LOCK_JAM = 9
 LOCK_STATE_UNLOCK_JAM = 10
+
+COMMAND_STATUS_PROCESSING = 4
+COMMAND_STATUS_DELIVERED = 7
+COMMAND_STATUS_CANCELLED = 6
+COMMAND_STATUS_COMPLETE = 5
